local fo = require("mods").requireFromPlugin("helpers.field_options")
local drawableSprite = require("structs.drawable_sprite")
local frostUtils = require("mods").requireFromPlugin("libraries.frostUtils")
local seamlessSpinner = {}

seamlessSpinner.name = "ChroniaHelper/SeamlessSpinner"
seamlessSpinner.placements = {
    name = "SeamlessSpinner",
    data = {
        foreDirectory = "blue",
        backDirectory = "blue",
        attachToSolid = false,
        hitboxType = "default",
        dust = false,
        customHitbox = "r,16,16,-8,-8;c,8,-8,-8",
        colorOverlay = "ffffff",
        depth = -8500,
        solidTileCutoff = true,
        noBorder = false,
        bloomAlpha = 0.0,
        bloomRadius = 0.0,
    },
}
seamlessSpinner.depth = function(room,entity) return entity.depth or -8500 end
seamlessSpinner.fieldInformation = {
    foreDirectory = {
        options={
            "Blue",
            "Red",
            "Purple",
            "Rainbow",
        },
        editable = true,
    },
    backDirectory={
        options={
            "Blue",
            "Red",
            "Purple",
            "Rainbow",
        },
        editable = true,
    },
    hitboxType={
        options={
            "default",
            "loosened",
            "seamlessRound",
            "seamlessSquare",
            "custom",
        },
        editable = false,
    },
    depth = fo.depths,
    customHitbox = {
        fieldType = "list",
        elementSeparator = ";",
    },
    colorOverlay = {
        fieldType = "color",
    },
}

function chooseSprite(entity)
    if entity.foreDirectory == "blue" or entity.foreDirectory == "Blue" then
        return "danger/crystal/fg_blue00"
    end
    if entity.foreDirectory == "red" or entity.foreDirectory == "Red" then
        return "danger/crystal/fg_red00"
    end
    if entity.foreDirectory == "Purple" or entity.foreDirectory == "purple" then
        return "danger/crystal/fg_purple00"
    end
    if entity.foreDirectory == "white" or entity.foreDirectory == "White" then
        return "danger/crystal/fg_white00"
    end
    if entity.foreDirectory == "Rainbow" or entity.foreDirectory == "rainbow" then
        return "danger/crystal/fg_white00"
    end
    return entity.foreDirectory .. "00"
end

seamlessSpinner.sprite = function(room,entity)
    local sprite = drawableSprite.fromTexture(chooseSprite(entity), entity)
	sprite:setColor(frostUtils.getColor(entity.colorOverlay))
	return sprite
end

seamlessSpinner.selection = function(room, entity)
    return require('utils').rectangle(entity.x - 8, entity.y - 8, 16, 16)
end

return seamlessSpinner